#!/usr/bin/env python2
# -*- coding: utf-8 -*-
# PYTHON_ARGCOMPLETE_OK

## Dependencies:    mplayer, argparse, argcomplete, pyYAML
## Author:          Gijs Timmers: https://github.com/GijsTimmers

## Licence:         CC-BY-SA-4.0
##                  http://creativecommons.org/licenses/by-sa/4.0/

## This work is licensed under the Creative Commons
## Attribution-ShareAlike 4.0 International License. To  view a copy of
## this license, visit http://creativecommons.org/licenses/by-sa/4.0/ or
## send a letter to Creative Commons, PO Box 1866, Mountain View,
## CA 94042, USA.

import subprocess               ## Om programma's uit te voeren vanuit Python
import sys                      ## Basislib
import os                       ## Basislib
import re                       ## Regex

class Radio():
    def __init__(self):
        if os.name == "posix":
            self.cmd = "mplayer"
        else:
            self.cmd = "mplayer.exe"
        
    def afspelen(self, zender, url, co):
        self.co = co
        try:        
            self.stream = subprocess.Popen([self.cmd, url], \
            stdin=subprocess.PIPE, \
            stdout=subprocess.PIPE, \
            stderr=subprocess.STDOUT, \
            bufsize=1, \
            universal_newlines=True)
            
        except OSError:
            print "Kon geen mplayer-executable vinden in $PATH."
            print "Installeer deze eerst:"
            print "Ubuntu:  sudo apt-get install mplayer"
            print "Arch:    sudo pacman -S mplayer"
            print "Windows: http://sourceforge.net/projects/mplayer-win32/"
            sys.exit() ## Moet nog aan gewerkt worden
        
        ## We encoderen de zendernaam in UTF-8 om errors te voorkomen
        ## in de stringnaam: "België" zou anders een probleem geven.
        self.co.processChannelName(zender.encode("utf-8"))
        
        
        for regel in iter(self.stream.stdout.readline, ''):
            ## Per nieuwe entry in stdout.readline wordt door deze loop
            ## gegaan. Als bijvoorbeeld de ICY-info verandert, wordt er
            ## opnieuw geprint: ICY Info: ... Dat wordt opgepakt door de if,
            ## en doorgegeven aan de communicator. Zo hebben we iedere keer
            ## de recentste info te pakken.
            
            if re.match("^ICY", regel):
                regel = re.findall(
                "(?<=ICY Info: StreamTitle=').*?(?=';)", regel
                                        )[0].strip()[:64]
                self.co.processIcy(regel)
                
            if re.match("^Exiting...", regel):
                ## Op een nieuwe regel starten
                sys.stdout.write("\n")
                break
        return()
        
        
        
    def stoppen(self):
        ## Terminaltitel opnieuw instellen op "Terminal"
        sys.stdout.write("\x1b]2;Terminal\x07")
        
        ## Stuur de toetsindruk Q naar de stream. mplayer reageert op q
        ## door te stoppen. Bij het stoppen print mplayer "Exiting...". In
        ## de thread t loopt een for-loop, welke deze string opvangt. Als 
        ## reactie stopt de for-loop ('break'), en komen we aan bij het
        ## return()-statenment. De thread is nu beëindigd.
        try:
            self.stream.communicate(input="q")
        
        ## IOError ontstaat soms door een deadlock in subprocess. Ik weet niet
        ## precies hoe ik die moet oplossen, daarom vang ik hem gewoon op 
        ## zonder er iets mee te doen.
        except IOError:
            sys.stdout.write("\n")



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
# PYTHON_ARGCOMPLETE_OK

## Dependencies:    mplayer, argparse, argcomplete, pyYAML
## Author:          Gijs Timmers: https://github.com/GijsTimmers

## Licence:         CC-BY-SA-4.0
##                  http://creativecommons.org/licenses/by-sa/4.0/

## This work is licensed under the Creative Commons
## Attribution-ShareAlike 4.0 International License. To  view a copy of
## this license, visit http://creativecommons.org/licenses/by-sa/4.0/ or
## send a letter to Creative Commons, PO Box 1866, Mountain View,
## CA 94042, USA.

import argcomplete              ## Argumenten aanvullen 
import argparse                 ## Parst argumenten
import yaml                     ## Configuratie inlezen
import os                       ## Basislib
import re                       ## Regex

class Parser():
    def __init__(self):
        ## Zenderdictionary aanmaken
        DEFAULT_YAML = os.path.join(os.path.dirname(__file__), "zenders.yml")
        CUSTOM_YAML  = os.path.expanduser("~/.yamlradio.yml")
        
        if os.path.isfile(CUSTOM_YAML):
            ## Load a user-definded yaml file
            loaded_yaml = CUSTOM_YAML
        else:
            ## Load the included yaml file
            loaded_yaml = DEFAULT_YAML
        
        with open(loaded_yaml, "r") as f:
            self.zenderdict = yaml.load(f)
        
        ## Afkortingenlijsten vullen met afkortingen en evt. namen
        self.afkortingenlijst = [combinatie["afk"] \
        for combinatie in self.zenderdict]
        self.afkortingenennamenlijst = [(combinatie["afk"], combinatie ["naam"]) \
        for combinatie in self.zenderdict]
        
        ## Parser instantiëren en de te verwachten argumenten meegeven
        self.parser = argparse.ArgumentParser(usage=self.helpoutput(), add_help=False)
        self.parser.add_argument('zender', choices=self.afkortingenlijst, help="hulp hier")
        
    def zendervinden(self):
        ## De ingevoerde argumenten parsen
        argcomplete.autocomplete(self.parser)
        argumenten = self.parser.parse_args()
        for combinatie in self.zenderdict:
            if combinatie["afk"] == argumenten.zender:
                naam = combinatie["naam"]
                url  = combinatie["url"]
                
                if re.match("\d", combinatie["afk"][0]):
                    comm = "_" + combinatie["afk"]
                else:
                    comm = combinatie["afk"]
                    
        return (naam, url, comm)    
    
    def helpoutput(self, name=None):
        ## We schrijven een eigen helpoutput, omdat die van argparse hier niet
        ## volstaat. De reden:
        ## We willen dat gebruikers "rd 3fm" kunnen schrijven ipv bvb
        ## "rd --3fm";
        ## De afwezigheid van streepjes impliceert een positioneel argument,
        ## ipv een optioneel argument;
        ## Daarom moeten we werken met één argument, genaamd "zender", welke
        ## meerdere keuzes heeft (de lijst van zenders);
        ## Dat leidt tot een kleine helpoutput, iets als "rd zender {}" met
        ## tussen de accolades alle mogelijke zenderafkortingen.
        SPACING = 2
        
        text = "rd [channel_abbreviation]\n" + \
        "available channels:\n"
        abbreviation_column_width = max([len(afk) for afk in self.afkortingenlijst]) + SPACING
        
        for c in self.afkortingenennamenlijst:
            text += ("  " + \
            c[0] + \
            ((abbreviation_column_width - len(c[0])) * " " ) + \
            c[1] + \
            "\n")
        return text


from setuptools import setup, find_packages
setup(
  name = "yamlradio",
  packages = ["yamlradio"],
  version = "1.4.2",
  description = "A small Python package to play radio stations as defined in a YAML file.",
  author = "Gijs Timmers",
  author_email = "gijs.timmers@student.kuleuven.be",
  url = "https://github.com/GijsTimmers/yamlradio",
  keywords = ["radio", "terminal", "yaml"],
  install_requires = ["argparse", "argcomplete", "pyYAML", "py-getch", "cursor"],
  classifiers = [],
  entry_points = {
        'console_scripts': ['rd=yamlradio:rd']},
  include_package_data = True
)



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
# PYTHON_ARGCOMPLETE_OK

## Dependencies:    mplayer, argparse, argcomplete, pyYAML
## Author:          Gijs Timmers: https://github.com/GijsTimmers

## Licence:         CC-BY-SA-4.0
##                  http://creativecommons.org/licenses/by-sa/4.0/

## This work is licensed under the Creative Commons
## Attribution-ShareAlike 4.0 International License. To  view a copy of
## this license, visit http://creativecommons.org/licenses/by-sa/4.0/ or
## send a letter to Creative Commons, PO Box 1866, Mountain View,
## CA 94042, USA.

import default                  ## Superklasse
import sys                      ## Basislib
import re                       ## Regex

class Communicator(default.Communicator):
    def processIcy(self, regel):
        ## Het oudeInfo/nieuweInfo-mechanisme
        ## is een mechanisme om iedere keer alleen het nieuwste ICY-bericht
        ## in het leesvenster te plaatsen.
        ## Wat uitleg over de regex:
        ## Alles tussen
        ## ICY Info: StreamTitle='
        ## en
        ## ';
        ## wordt opgeslagen als nieuweInfo. .* is non-greedy gemaakt
        ## met een vraagteken, zodat een eventueel volgende streamUrl
        ## niet wordt opgenomen in de nieuweInfo. Uiteindelijk nog een
        ## strip()-statement om losse spaties voorin en achterin de
        ## string weg te nemen.
        
        self.nieuweInfo = regel
        if self.nieuweInfo != self.oudeInfo:
            if " - " in self.nieuweInfo:
                ## Als er een liedje wordt afgespeeld (gekenmerkt door het
                ## streepje), wordt de string omgezet in kleine letters, be-
                ## ginnend met een hoofdletter.
                self.nieuweInfo = self.nieuweInfo.title()
            
            sys.stdout.write("\r" + " " * self.BREEDTE_TERMINAL)
            sys.stdout.write("\r" + "Info:         [{info}]".format(info=self.nieuweInfo))
            self.oudeInfo = self.nieuweInfo